########### FUNCTIONS ---------------------------------------------------------------------

using DataFrames
using SpecialFunctions

SPECIES = 1
MORPH_ALLELE1 = 2
MORPH_ALLELE2 = 3
MORPH = 4
RESUSE_ALLELE1 = 5
RESUSE_ALLELE2 = 6
RESUSE = 7
RESUSEHETERO = 8

NICHE1 = 1
NICHE2 = 2


# CREATE OBJECTS

function createCompetitionFunctionMatrix()
  global CompetitionFunctionMatrix = zeros(Float32,(sizeCompetitionFunctionMatrix+1)^4)
  interNiche = 1.0/sizeCompetitionFunctionMatrix

  index = 1

  niche1 = 0.0
  while niche1<=1.001
    niche1Bis = 0.0
    while niche1Bis<=1.001
      niche2 = 0.0
      while niche2<=1.001
	niche2Bis = 0.0
	while niche2Bis<=1.001
	  #CompetitionFunctionMatrix[index] = niche2Bis
	  CompetitionFunctionMatrix[index] = exp( - ( (niche1-niche1Bis)^2+
						    (niche2-niche2Bis)^2  ) / (2 * sigmaC^2) )
	  index = index+1
	  niche2Bis=niche2Bis+interNiche
	end
	niche2=niche2+interNiche
      end
      niche1Bis=niche1Bis+interNiche
    end
    niche1=niche1+interNiche
  end
end

function getIndexCompetitionFunctionMatrix(niche1, niche2, niche1Bis, niche2Bis)
  interNiche = 1.0/sizeCompetitionFunctionMatrix
  sizeCompetitionFunctionMatrix
  
  niche1Index::Int32 = round((round(niche1*sizeCompetitionFunctionMatrix)/sizeCompetitionFunctionMatrix)/interNiche)+1
  niche1BisIndex::Int32 = round((round(niche1Bis*sizeCompetitionFunctionMatrix)/sizeCompetitionFunctionMatrix)/interNiche)+1
  niche2Index::Int32 = round((round(niche2*sizeCompetitionFunctionMatrix)/sizeCompetitionFunctionMatrix)/interNiche)+1
  niche2BisIndex::Int32 = round((round(niche2Bis*sizeCompetitionFunctionMatrix)/sizeCompetitionFunctionMatrix)/interNiche)+1

  index::Int32 = (sizeCompetitionFunctionMatrix+1)^3 * (niche1Index-1) + 
		 (sizeCompetitionFunctionMatrix+1)^2 * (niche1BisIndex-1) +
		 (sizeCompetitionFunctionMatrix+1) * (niche2Index-1) +
		 (niche2BisIndex) 
  return index
end

function createPopulation()
  # row = ind
  # column = characteristics
  global PopListDiscrete = zeros(Int8,0,8)
  global PopListContinuous = zeros(Float32,0,2)
  global MorphDensityList = zeros(Int32,Nmorph)
  global ResUseDensityList = zeros(Int32,Nresource)
  global SizePop = 0

  PopListDiscreteOneInd = zeros(Int8,1,8)
  PopListContinuousOneInd = zeros(Float32,1,2)

  morphUsed::Int8 = 1
  ResUseUsed::Int8 = 1
  vecMorphUsed = zeros(Int32,Nspecies)
  vecResUsed = zeros(Int32,Nspecies)
  for sp in 1:Nspecies
    vecMorphUsed[sp] = morphUsed
    vecResUsed[sp] = ResUseUsed
    morphUsed = morphUsed + 1
    if morphUsed>Nmorph
      morphUsed = 1
    end
    ResUseUsed = ResUseUsed + 1
    if ResUseUsed>Nresource
      ResUseUsed = 1
    end     
  end
  # we randomize characteristics of each species
  vecMorphUsed = vecMorphUsed[randperm(length(vecMorphUsed))]
  vecResUsed = vecResUsed[randperm(length(vecResUsed))]

  for sp::Int8 in 1:Nspecies

    for ind in 1:Nini
    
      #morphUsed = rand(1:Nmorph)	#if random ind
      #ResUseUsed = rand(1:Nresource)	#if random ind

      PopListDiscreteOneInd[SPECIES] = sp
      PopListDiscreteOneInd[MORPH_ALLELE1] = vecMorphUsed[sp]; PopListDiscreteOneInd[MORPH_ALLELE2] = vecMorphUsed[sp]
      PopListDiscreteOneInd[MORPH] = vecMorphUsed[sp]

      PopListDiscreteOneInd[RESUSE_ALLELE1] =  vecResUsed[sp]; PopListDiscreteOneInd[RESUSE_ALLELE2] = vecResUsed[sp]
      PopListDiscreteOneInd[RESUSE] = vecResUsed[sp]

      PopListContinuousOneInd[NICHE1] = rand(Float32)
      PopListContinuousOneInd[NICHE2] = rand(Float32)

      PopListDiscrete = vcat(PopListDiscrete, PopListDiscreteOneInd)
      PopListContinuous = vcat(PopListContinuous, PopListContinuousOneInd)
      MorphDensityList[vecMorphUsed[sp]] = MorphDensityList[vecMorphUsed[sp]] + 1
      SizePop = SizePop + 1
    end  
  end

  global CompetitionTermMatrix = zeros(Float32,size(PopListDiscrete,1),size(PopListDiscrete,1))
  global DensityBetaProbabilitiesList = zeros(Float32,size(PopListDiscrete,1),Npred)
  
  for ind1 in 1:size(PopListDiscrete,1)
    for ind2 in 1:size(PopListDiscrete,1)

	CompetitionTermMatrix[ind1, ind2] = exp( - ( (PopListContinuous[ind1,NICHE1]-PopListContinuous[ind2,NICHE1])^2+
						  (PopListContinuous[ind1,NICHE2]-PopListContinuous[ind2,NICHE2])^2  ) / (2 * sigmaC^2) )

    end  
    for pred in 1:Npred
      DensityBetaProbabilitiesList[ind1, pred] = DensityBetaProbability(PopListContinuous[ind1,NICHE1], PopListContinuous[ind1,NICHE2],
								        pred)
    end
  end  
end



# FUNCTIONS USED IN THE SIMULATION

function determineIndexOfPreyEatenGompert()
  listPred = 1:Npred
  listIndEaten = zeros(Int32,Npred*Nmorph*nk)
  indexVector1 = 1
  for morphAttacked in 1:Nmorph
    if MorphDensityList[morphAttacked]>nk*Npred
      
      #list of individuals with the given morph
      IndexIndList = zeros(Int32,MorphDensityList[morphAttacked])
      indexVector2 = 1
      for ind in 1:size(PopListDiscrete,1)
        if PopListDiscrete[ind,MORPH] == morphAttacked
            IndexIndList[indexVector2] = ind
            indexVector2 = indexVector2 + 1
        end
      end    
      listPred = listPred[randperm(length(listPred))]
      for predIndex in listPred 
            ProbaPredationList = zeros(Float32,size(IndexIndList,1))    
            indexVector2 = 1
            for ind in IndexIndList
                ProbaPredationList[indexVector2] = DensityBetaProbabilitiesList[ind, predIndex]
                indexVector2 = indexVector2 + 1
            end    

            for attackEvent in 1:nk
                index = determineIndexFromCumulativeRates(ProbaPredationList)
                listIndEaten[indexVector1] = IndexIndList[index]
	  
                ProbaPredationList = ProbaPredationList[setdiff(1:end, index)]
                IndexIndList = IndexIndList[setdiff(1:end, index)]
                indexVector1 = indexVector1 + 1
            end
        end
        else
            for ind in 1:size(PopListDiscrete,1)
                if PopListDiscrete[ind,MORPH] == morphAttacked
                    listIndEaten[indexVector1] = ind
                    indexVector1 = indexVector1 + 1
                end
            end
        end
    end
  return listIndEaten
end


function determineIndexOfPreyEaten()
  listPred = 1:Npred
  listMorph = 1:Nmorph
  listIndEaten = zeros(Int32,Npred*Nmorph*nk)
  indexVector1 = 1
  
  IndexIndList = 1:size(PopListDiscrete,1)
  
  listPred = listPred[randperm(length(listPred))]
  for predIndex in listPred 
     # proba of predator type to see each prey
    IndexIndListCopy = copy(IndexIndList)
    ProbaPredationList = zeros(Float32,size(IndexIndListCopy,1))   
    for ind in 1:length(IndexIndListCopy)
      ProbaPredationList[ind] = DensityBetaProbabilitiesList[IndexIndListCopy[ind], predIndex]
    end
     # prey that the predator type sees
    nbPrey = length(IndexIndListCopy)
    listPreyVisible = zeros(Int32,min(Nindpred,nbPrey))
    for indpred in 1:min(Nindpred,nbPrey)
      index = determineIndexFromCumulativeRates(ProbaPredationList)     
      listPreyVisible[indpred] = IndexIndListCopy[index]
      ProbaPredationList = ProbaPredationList[setdiff(1:end, index)]
      IndexIndListCopy = IndexIndListCopy[setdiff(1:end, index)]
    end    

     # attack on each morph
    listMorph = listMorph[randperm(length(listMorph))]
#     x1M = 0
#     x2M = 0
#     nM = 0
    for morphAttacked in listMorph
      listPreyVisible = listPreyVisible[randperm(length(listPreyVisible))]
      nattack = 0
      indexVector4 = 1
      while nattack<nk && indexVector4<=length(listPreyVisible)
	ind = listPreyVisible[indexVector4]	  
	indexVector4 = indexVector4 + 1
	if PopListDiscrete[ind,MORPH] == morphAttacked
	  nattack = nattack + 1
	  listIndEaten[indexVector1] = ind
	  indexVector1 = indexVector1 + 1 
	  indexVector5 = 1
	  indRemove = -1
	  while indexVector5<=length(IndexIndList) && indRemove == -1
	    if IndexIndList[indexVector5] == ind
	      indRemove = copy(indexVector5)
	    end
	    indexVector5 = indexVector5 + 1
	  end
	  IndexIndList = IndexIndList[setdiff(1:end, indRemove)]	  
	end
      end
    end 
  end
  sav = 0
  found = 0
  for i in 1:length(sort(listIndEaten))
    if sort(listIndEaten)[i]!=0
      if sort(listIndEaten)[i] == sav
	found = 1
      end
      sav = sort(listIndEaten)[i]
    end
  end
  if found==1
    println("same individual eaten twice")
  end
  return listIndEaten
end


function determineIndexOfPreyDying()
  listIndDying = zeros(Int32,size(PopListDiscrete,1))
  
  indexVector1 = 1
  for ind in 1:size(PopListDiscrete,1)
    
    resourceUsed = PopListDiscrete[ind,RESUSE] 
    speciesInd = PopListDiscrete[ind,SPECIES] 

    cond = 1.0
    if sigmaK<5.0
        cond = cond * exp(- ((PopListContinuous[ind,NICHE1]-KOptVec[resourceUsed,1])^2+(PopListContinuous[ind,NICHE2]-KOptVec[resourceUsed,2])^2)/(2*sigmaK^2))
    end
    
      sumCompet = 0
      for ind2 in 1:size(PopListDiscrete,1)
	if PopListDiscrete[ind2,RESUSE] == resourceUsed #same resources
	  if PopListDiscrete[ind2,SPECIES] == speciesInd
	    sumCompet = sumCompet + CompetitionTermMatrix[ind, ind2]
	  else
	    sumCompet = sumCompet + CompetitionTermMatrix[ind, ind2] * alphaCompetDiffSpecies # * alphaCompetDiffSpecies   # 0.1
	  end
	else #different resources
	  if PopListDiscrete[ind2,SPECIES] == speciesInd
	    sumCompet = sumCompet + CompetitionTermMatrix[ind, ind2] * alphaCompetDiffRes # * alphaCompetDiffRes   # 0.1
	  else
	    sumCompet = sumCompet + CompetitionTermMatrix[ind, ind2] * alphaCompetDiffRes * alphaCompetDiffSpecies # * alphaCompetDiffRes * alphaCompetDiffSpecies   # 0.
	  end
	end
      end

    
    if sigmaK<5.0 
        viability = 1 / ( 1 + (b-1)*sumCompet/(cond*betaHetRes*Kmax) )
    else
        viability = 1 / ( 1 + (b-1)*sumCompet/(cond*Kmax) )
    end    

    if rand()>viability
      listIndDying[indexVector1] = ind
      indexVector1 = indexVector1 + 1
    end
  end
  
  return listIndDying
end

function birthIndividuals()

  global PopListDiscrete
  global PopListContinuous 
  global CompetitionTermMatrix 
  global DensityBetaProbabilitiesList 
  global MorphDensityList 
  global ResUseDensityList
  global SizePop 
  

  SizePopParents = copy(SizePop)
  SizePop = SizePop*b #offspring
  
  SpeciesIndexesIndividuals = zeros(Int32,Nspecies,SizePopParents)
  nbIndividualsPerSpecies = zeros(Int32,Nspecies)

  for sp::Int8 in 1:Nspecies
    nbOneSp = 0
    for ind in 1:SizePopParents
        if PopListDiscrete[ind,SPECIES] == sp
            nbOneSp = nbOneSp + 1
            SpeciesIndexesIndividuals[sp,nbOneSp] = ind
        end
    end
    nbIndividualsPerSpecies[sp] = nbOneSp
  end

  PopListDiscreteOffspring = zeros(Int8,0,8)
  PopListContinuousOffspring = zeros(Float32,0,2)


  NormalDistribution = Normal(0, sigmaNiche)

  vecSp = 1:Nspecies
  vecSp = vecSp[randperm(length(vecSp))]

  for sp::Int8 in vecSp 
    for ind in 1:SizePopParents
        if PopListDiscrete[ind,SPECIES] == sp
            # what mate?
            randMateIndex = SpeciesIndexesIndividuals[sp,rand(1:nbIndividualsPerSpecies[sp])]
            foundMate = matingOrRejection(ind,randMateIndex)
            while foundMate == 0 
            randMateIndex = SpeciesIndexesIndividuals[sp,rand(1:nbIndividualsPerSpecies[sp])]
            foundMate = matingOrRejection(ind,randMateIndex)
            end
            for offspring in 1:b
            PopListContinuousOneOffspring = zeros(Float32,1,2)
            PopListDiscreteOneOffspring = zeros(Int8,1,8)
            
            PopListDiscreteOneOffspring[SPECIES] = sp
            
                #### MORPH AND RESUSE
            if rand()<0.5
                PopListDiscreteOneOffspring[MORPH_ALLELE1] = PopListDiscrete[ind,MORPH_ALLELE1] # mother
            else
                PopListDiscreteOneOffspring[MORPH_ALLELE1] = PopListDiscrete[randMateIndex,MORPH_ALLELE1] # father
            end
            
            if rand()<0.5
                PopListDiscreteOneOffspring[RESUSE_ALLELE1] = PopListDiscrete[ind,RESUSE_ALLELE1] # mother
            else
                PopListDiscreteOneOffspring[RESUSE_ALLELE1] = PopListDiscrete[randMateIndex,RESUSE_ALLELE1] # father
            end
            
            #mutations
            if rand()<mMorph
                PopListDiscreteOneOffspring[MORPH_ALLELE1] =  rand(setdiff(1:Nmorph, PopListDiscreteOneOffspring[MORPH_ALLELE1]))
            end
            if rand()<mRessource
                PopListDiscreteOneOffspring[RESUSE_ALLELE1] =  rand(setdiff(1:Nresource, PopListDiscreteOneOffspring[RESUSE_ALLELE1]))
            end

            PopListDiscreteOneOffspring[MORPH] = PopListDiscreteOneOffspring[MORPH_ALLELE1]	      
            PopListDiscreteOneOffspring[RESUSE] = PopListDiscreteOneOffspring[RESUSE_ALLELE1]	      

                #### NICHE USE	
            saveNiche1 = mean([PopListContinuous[ind,NICHE1] PopListContinuous[randMateIndex,NICHE1]]) # mean parents
            while true
                PopListContinuousOneOffspring[NICHE1] = saveNiche1 + rand(NormalDistribution)
                (PopListContinuousOneOffspring[NICHE1]>= 0)&&(PopListContinuousOneOffspring[NICHE1]<=1) && break
            end
            saveNiche2 = mean([PopListContinuous[ind,NICHE2] PopListContinuous[randMateIndex,NICHE2]])   
            while true
                PopListContinuousOneOffspring[NICHE2] = saveNiche2 + rand(NormalDistribution)
                (PopListContinuousOneOffspring[NICHE2]>= 0)&&(PopListContinuousOneOffspring[NICHE2]<=1) && break
            end
            
            PopListDiscreteOffspring = vcat(PopListDiscreteOffspring, PopListDiscreteOneOffspring)
            PopListContinuousOffspring = vcat(PopListContinuousOffspring, PopListContinuousOneOffspring)

            end	    
            end
        end
    end
  PopListDiscrete = copy(PopListDiscreteOffspring)
  PopListContinuous = copy(PopListContinuousOffspring)        
    

  MorphDensityList = zeros(Int32,Nmorph)
  ResUseDensityList = zeros(Int32, Nresource)
  CompetitionTermMatrix = zeros(Float32,size(PopListDiscrete,1),size(PopListDiscrete,1))
  DensityBetaProbabilitiesList = zeros(Float32,size(PopListDiscrete,1),Npred)

  for ind1 in 1:size(PopListDiscrete,1)
    
    MorphDensityList[PopListDiscrete[ind1,MORPH]] = MorphDensityList[PopListDiscrete[ind1,MORPH]] + 1
    ResUseDensityList[PopListDiscrete[ind1,RESUSE]] = ResUseDensityList[PopListDiscrete[ind1,RESUSE]] + 1
      
    for ind2 in 1:size(PopListDiscrete,1)
	CompetitionTermMatrix[ind1, ind2] = exp( - ( (PopListContinuous[ind1,NICHE1]-PopListContinuous[ind2,NICHE1])^2+
						    (PopListContinuous[ind1,NICHE2]-PopListContinuous[ind2,NICHE2])^2  ) / (2 * sigmaC^2) )
    end  
    for pred in 1:Npred
      DensityBetaProbabilitiesList[ind1, pred] = DensityBetaProbability(PopListContinuous[ind1,NICHE1], PopListContinuous[ind1,NICHE2],
								        pred)
    end
  end   
end

function matingOrRejection(ind1,ind2)
  matingOccur = 0
  prob = exp(-choosinessStrength^2 * ((PopListContinuous[ind1,NICHE1]-PopListContinuous[ind2,NICHE1])^2+
                                        (PopListContinuous[ind1,NICHE2]-PopListContinuous[ind2,NICHE2])^2))
  if prob<0.00001
    prob=0.00001
  end
  if rand()<prob
    matingOccur = 1
  end
  return matingOccur
end

function deathIndividuals(listDead)

  global PopListDiscrete
  global PopListContinuous 
  global CompetitionTermMatrix 
  global DensityBetaProbabilitiesList 
  global MorphDensityList 
  global ResUseDensityList 
  global SizePop 
  
  indexStop = 0
  foundZero = 0
  listDead = sort(listDead,rev=true)
  while indexStop < size(listDead,1) && foundZero==0
    indexStop = indexStop + 1
    if listDead[indexStop]==0
      foundZero = 1
    end
  end
  if foundZero==1
    listDead = listDead[1:(indexStop-1)]
  end
  SizePop = SizePop - size(listDead,1)

  for deadInd in listDead
    MorphDensityList[PopListDiscrete[deadInd,MORPH]] = MorphDensityList[PopListDiscrete[deadInd,MORPH]] - 1
    ResUseDensityList[PopListDiscrete[deadInd,RESUSE]] = ResUseDensityList[PopListDiscrete[deadInd,RESUSE]] - 1
  end

  PopListDiscrete = PopListDiscrete[setdiff(1:size(PopListDiscrete,1),listDead),1:end]
  PopListContinuous = PopListContinuous[setdiff(1:end, listDead),1:end]
  CompetitionTermMatrix = CompetitionTermMatrix[setdiff(1:end, listDead),setdiff(1:end, listDead)]

  DensityBetaProbabilitiesList = DensityBetaProbabilitiesList[setdiff(1:end, listDead),1:end]
end



# MAIN FUNCTION TO RUN SIMULATION

function SingleTimeSeries(nameFile, Tmax, nbRecord)

  global PopListDiscrete
  global PopListContinuous 
  global CompetitionTermMatrix 
  global DensityBetaProbabilitiesList 
  global MorphDensityList 
  global ResUseDensityList
  global SizePop 
  
  if isfile("Data/SingleTimeSeries"*nameFile*".txt")==true
    rm("Data/SingleTimeSeries"*nameFile*".txt")
  end
  file = open("Data/SingleTimeSeries"*nameFile*".txt", "w")

  write(file,"time;sp;morph;resUse;niche1;niche2 \n")
  

  Tinterrecord = round(Tmax/nbRecord)
  Tnextrecord = copy(Tinterrecord)
  println(Tinterrecord)

  t=0
  
  #createCompetitionFunctionMatrix()
  createPopulation()
  
  for ind in 1:size(PopListDiscrete,1)
    write(file,"$(t);$(PopListDiscrete[ind,SPECIES]);$(PopListDiscrete[ind,MORPH]);$(PopListDiscrete[ind,RESUSE]);$(PopListContinuous[ind,NICHE1]);$(PopListContinuous[ind,NICHE2]) \n")
  end
  
  println("start")
  while true

    # PREDATION
    if nk>0
      listIndEaten = determineIndexOfPreyEaten()
      deathIndividuals(listIndEaten)
    end
    # BIRTH
    birthIndividuals()
    
    # DEATH: LOGISTIC GROWTH
    listIndDying = determineIndexOfPreyDying()
    deathIndividuals(listIndDying)

    
    if t>=Tnextrecord
      println(Tnextrecord)
      println(size(PopListDiscrete,1))
      for ind in 1:size(PopListDiscrete,1)
	write(file,"$(Tnextrecord);$(PopListDiscrete[ind,SPECIES]);$(PopListDiscrete[ind,MORPH]);$(PopListDiscrete[ind,RESUSE]);$(PopListContinuous[ind,NICHE1]);$(PopListContinuous[ind,NICHE2]) \n")
      end
      Tnextrecord = Tnextrecord + Tinterrecord
    end    
    
    t = t + 1
    
    t>=(Tmax+0.1) && break
  end  
  
  close(file)
end




# OTHER FUNCTIONS

function determineIndexFromCumulativeRates(RateVec)
  CumulativeRateVec = copy(RateVec)
  for i in 2:length(CumulativeRateVec)
    CumulativeRateVec[i] = CumulativeRateVec[i-1] + CumulativeRateVec[i]
  end
  CumulativeRateVec = CumulativeRateVec/CumulativeRateVec[length(CumulativeRateVec)]

  randNb = rand()

  indexEvent = 1
  found = false
  while indexEvent<length(CumulativeRateVec) && found == false
    if randNb<CumulativeRateVec[indexEvent]
      found = true
    else
      indexEvent=indexEvent+1
    end
  end
  return indexEvent
end


function DensityBetaProbability(x1, x2, pred)
  Constant1 = gamma(ParamPredationVec[pred,1]+ParamPredationVec[pred,2])/(gamma(ParamPredationVec[pred,1])*gamma(ParamPredationVec[pred,2]))
  Constant2 = gamma(ParamPredationVec[pred,3]+ParamPredationVec[pred,4])/(gamma(ParamPredationVec[pred,3])*gamma(ParamPredationVec[pred,4]))
  densityProba::Float32 = Constant1*x1^(ParamPredationVec[pred,1]-1)*(1-x1)^(ParamPredationVec[pred,2]-1) * 
		 Constant2*x2^(ParamPredationVec[pred,3]-1)*(1-x2)^(ParamPredationVec[pred,4]-1)
  return densityProba
end

function checkAll()
  println("")
  println(SizePop)
  println(size(PopListDiscrete))
  println(size(PopListContinuous))
  println(size(CompetitionTermMatrix))
  println(size(DensityBetaProbabilitiesList))
  println(sum(MorphDensityList))
  println(sum(ResUseDensityList))
end
