include("./Functions.jl")

using Distributions
using SpecialFunctions
using Random

########### MAIN ---------------------------------------------------------------------

const Nresource = 4     # number of resources
const Nmorph = 10       # number of morphs
const Npred = 4         # number of predator communities

const Nspecies = 20     # number of species

const b = 2             # number of offspring per individual
const Kmax = 500        # maximum of the carrying capacity function (adjusted with betaHetRes, defined below)
const Nini = 250        # number of individuals initially

const Nindpred = 500    # number of prey encountered by each predator community per time step
const nk = 15           # number of prey of each morph eaten by each predator community per time step

const alphaCompetDiffSpecies = 0.05           # maximum competition term between different species
const alphaCompetDiffRes = 0.5                # maximum competition term on different resources

const sigmaC = 1.                             # Standard deviation of the competition kernel
const sizeCompetitionFunctionMatrix = 50      # To avoid the billions of calculation; calculation the strength of competition is calculated once (discretized in 50), and stored

const choosinessStrength = 0                  # assortative mating based on the ecological niche space
const choosinessStrengthMorph = 0.5           # assortative mating based on the morph

const sigmaNiche = 0.05                       # variance between parent and offspring micro-habitat use 
const mRessource = 0.0                        # mutation rate to an alternative resource
const mMorph = 0.01                           # mutation rate to an alternative morph

const Tmax = 1000                             # Number of time steps (runtime is very high; choose Tmax=100 for a faster run)
const nbRecord = 10                           # Number of snapshop of the ecological community within the simulation


const nameFile = "Simulation_SigmaK1PredHoRep1"     # Name of the simulation
sigmaK = 1.0                                        # standard deviation of the carrying capacity function; adjust the carrying capacity via betaHetRes (see below): 
                                                    # 1.0 (H-Het); 1.5 (M-Het) or 6.0 (Hom) in the present study
predationHet = "o"                                  # heterogeneity of predator micro-habitat use: homogeneous: "o" , moderately heterogeneous: "m" or highly heterogeneous: "e"  in the present study
repIndex = 1                                        # index of the replicate


#######################################################################################

const minNicheK = 0.
const maxNicheK = 1.

KOptVec = [  minNicheK  minNicheK ;         # position of the four resources optima 
             minNicheK  maxNicheK ;
             maxNicheK  minNicheK ;
             maxNicheK  maxNicheK ]


## change of the maximum carrying capacity depending on the heterogeneity of resources (sigmaK)
if sigmaK == 1.0
  betaHetRes = 1.15
elseif sigmaK == 1.5
  betaHetRes = 1.1
elseif sigmaK == 6
  betaHetRes = 1.15
end

## change of the parameters linked to predator micro-habitat use
if predationHet == "e"
  ParamPredation1 = 4.
  ParamPredation2 = 1.5
elseif predationHet == "m"
  ParamPredation1 = 2.
  ParamPredation2 = 1.
elseif predationHet == "o"
  ParamPredation1 = 1.0
  ParamPredation2 = 1.0
end
             
ParamPredationVec = [  ParamPredation1  ParamPredation2 ParamPredation1 ParamPredation2 ParamPredation1  ;
                       ParamPredation1  ParamPredation2 ParamPredation2 ParamPredation1 ParamPredation1  ;
                       ParamPredation2  ParamPredation1 ParamPredation2 ParamPredation1 ParamPredation1  ;
                       ParamPredation2  ParamPredation1 ParamPredation1 ParamPredation2 ParamPredation1  ]
                       



println(nameFile)
global PopListDiscrete
global PopListContinuous 
global CompetitionTermMatrix 
global DensityBetaProbabilitiesList 
global MorphDensityList 
global ResUseDensityList
global SizePop
global CompetitionFunctionMatrix

@time SingleTimeSeries(nameFile, Tmax, nbRecord)
